#!/usr/bin/python

import os, sys, shutil, ConfigParser, urllib, subprocess

def documentation():
		'''
		Arguments:
			None: do install servlets
		At install:
		- Creates servlets, directory, common (classes) and configuration files, 
		 -- reads and parses the input configuration files:
			 /etc/tomcat/servlets.conf
			 /etc/tomcat/servlets.passwd
			 /etc/tomcat/servlets.confConstants
		 -- Per servlet:
			o Creates a main directory for the servlet
			o Copies common (class) files under it
			o Creates 3 per servlet configuration files (using the unproto.sh for this, applying per conf env vars)

		For list of created files and structure of the config files: see: http://frontier.cern.ch/dist/rpms/_README

		TBD
		- Take care of multiple projects/sites as done at awstats
		- Esthetics:
				use args instead of env vars
				use class/es
		'''

debug=False
# Debug may be enforced, either by uncommenting the following comment, 
#   or by creating a "/tmp/frontier_servlet_debug" file
#debug=True # uncomment to debug 
if os.path.isfile( "/tmp/frontier_servlet_debug"):
	debug=True

def error( message):
	print 'ERROR:',message,'.'
	sys.exit(-1)

def run_cmd(cmd):
	p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
	output = p.communicate()[0]
	return output

# Common directories and files

tomcatSymbolicLink=  str(os.environ.get('CATALINA_HOME'))
tomcatBaseDir= tomcatSymbolicLink+'/../'
frontierUser= str(os.environ.get('UserName')) # default: dbfrontier

unprotoScript='unproto.sh'
unprotoSh= tomcatBaseDir+str(unprotoScript)

servletsConfigDir= '/etc/tomcat'
servletsConfigDir_= tomcatBaseDir+'etc'

servletsXmlDir= tomcatSymbolicLink+ '/conf/Catalina/localhost'

commonServletProto_= 'servlet.xml.proto'
commonServletProto= servletsConfigDir_+ '/' + commonServletProto_

commonConfigPropertiesProto_= 'config.properties.proto'
commonConfigPropertiesProto= servletsConfigDir_+ '/'+ commonConfigPropertiesProto_
commonConfigPropertiesSmallFilesProto_= 'config.propertiesSmallFiles.proto'
commonConfigPropertiesSmallFilesProto= servletsConfigDir_+ '/'+ commonConfigPropertiesSmallFilesProto_

commonWebXmlProto_= 'web.xml.proto'
commonWebXmlProto= servletsConfigDir_+ '/'+ commonWebXmlProto_

passwords= ConfigParser.ConfigParser()
servletsConstants= ConfigParser.ConfigParser()
servlets= ConfigParser.ConfigParser()

# Correction to support case sensitive configuration options (keys)
passwords.optionxform = servlets.optionxform = servletsConstants.optionxform= str 


passwordsFile= servletsConfigDir+ '/servlets.passwd'
servletsFile= servletsConfigDir+ '/servlets.conf'
#servletsConstantsFile= servletsConfigDir_+ '/tomcat/servlets.confConstants'
servletsConstantsFile= servletsConfigDir+ '/servlets.confConstants'
if debug:
	print 'servletsConstantsFile:',servletsConstantsFile
metaConfigFiles=[ passwordsFile, servletsFile, servletsConstantsFile]
# Could be implemented as a loop using a dict of files per config
servletsConstants.read( servletsConstantsFile)
passwords.read( passwordsFile)
servlets.read( servletsFile)
jarFile='ojdbc5_g.jar'

def mkdirIfMissing( path):
	if not os.path.isdir( path):
		os.makedirs( path) # makedirs is like mkdir -p, but fails if dir exists 

def printConfig( config, name):
	print 'start printConfig:', name
	for section in config.sections():
		print '['+section+']'
		for option in config.options(section):
			if debug:
				print option+':', config.get(section, option)
	print 'end: printConfig'

if debug:
	printConfig(passwords,'passwords')
	printConfig(servletsConstants,'servletsConstants')
	printConfig(servlets,'servlets')

envVarsToSet={}

# Copy rows of the following format from fromFile to toFile:
# <anything>=@@@<something that has a key at envVarsToSet>@@@
def copyProtoRowOfEnvVarsToSet( fromFileName, toFileName):
	fromFile= open(fromFileName)
	toFile= open(toFileName,'w')
	for line in fromFile:
		splitted=line.split('@@@')
		if len(splitted)>1:
			var=splitted[1]
			if envVarsToSet.has_key(var):
				toFile.write(line)
	fromFile.close()
	toFile.close()

def setEnvVars():
	for key in envVarsToSet.keys():
		os.environ[ key]= envVarsToSet[ key] 

constants=set()
def applyConfigSection( config, section):
	for option in config.options( section):
		# constants should not be repeated at servlets.conf
		# TBD: consider printing all repeating constants rather than the first one only
		if section=='constants':
			constants.add(option)
		else:
			if option in constants:
				error('Constant repeated at servlets.conf: "'+ option+ '"')

		value= str( config.get(section, option))
		if debug:
			print 'section:',section,', option:',option,', value:',value
		# options may need environment variables
		envVarsToSet[option]= value

def createServletXmlFile( servlet, files4proto):
	if servlet=='defaults':
		return

	# get and unzip the common servlet (classes) files
	# fail if  baseDir+'/tomcat' does not exist, because this may signify that the correct release of tomcat is not installed.
	# TBD: verify if the check is thorrough enough by removing the dir

	# Find or create directores
	if not os.path.isdir( tomcatSymbolicLink):
		error( 'Missing directory: '+ tomcatSymbolicLink)
	mkdirIfMissing( servletBaseDir)
	mkdirIfMissing( servletClassesDir)

	# wget (or cp) and unzip war file if missing
	os.chdir(servletBaseDir)
	warFilName= 'Frontier_3.29.war'
	if not os.path.isfile( warFilName):
		shutil.copyfile( tomcatBaseDir+warFilName, warFilName)
		#urlServletWar= 'http://frontier.cern.ch/dist/'+warFilName
		#urllib.urlretrieve(urlServletWar, warFilName)
		# TBD: consider: maybe needed even if war exists, in case of previous failure to unzip
		os.system('unzip '+warFilName+' >/dev/null') # Not nice, but python does not seem to support unzip easily

	# Get configuration from servlets and passwords files
	envVarsToSet['servlet']= servlet # Exception, needs manual setting
	if 'constants' in servletsConstants.sections():
		applyConfigSection( servletsConstants, 'constants')
	if 'defaults' in servlets.sections():
		applyConfigSection( servlets, 'defaults')
	applyConfigSection( servlets, servlet)

	# If there is no FileBaseDirectory entry:
	if not 'FileBaseDirectory' in envVarsToSet.keys():
		# create a servlet specific proto file
		servletProto= files4proto['servletXml']+'.proto'
		mkdirIfMissing( servletsXmlDir)
		shutil.copyfile( commonServletProto, servletProto) 

	# Create a web.xml.proto
	webXmlProto= files4proto['webXml']+'.proto'
	shutil.copyfile( commonWebXmlProto, webXmlProto)

	# If there is no FileBaseDirectory entry:
	if not 'FileBaseDirectory' in envVarsToSet.keys():
		# The key of the password may be either tnsName or username 
		tnsName= envVarsToSet['tnsName']
		username= envVarsToSet['username']
		if debug:
			print 'tnsName:',tnsName
			print 'username:',username
		try:
			applyConfigSection( passwords, tnsName)
		except:
			applyConfigSection( passwords, username)

		# Create a config.properties.proto
		configPropertiesProto= files4proto['configProperties']+'.proto'
		copyProtoRowOfEnvVarsToSet( commonConfigPropertiesProto, configPropertiesProto)
		protoFiles= [servletProto, configPropertiesProto, webXmlProto]
	else:
		configPropertiesSmallFilesProto= files4proto['configPropertiesSmallFiles']+'.proto'
		copyProtoRowOfEnvVarsToSet( commonConfigPropertiesSmallFilesProto, configPropertiesSmallFilesProto)
		protoFiles= [configPropertiesSmallFilesProto, webXmlProto]

	# Set configuration variables as environment variables
	setEnvVars()

	# Apply protos
	mode='600'
	envVarNames= str(envVarsToSet.keys()).replace('[','').replace(']','').replace(',','').replace("'",'') # TBD: find more clean syntax
	doEraseProto='0' # Do not erase protofiels after using them
	for proto in protoFiles:
		cmd= unprotoSh+ ' '+ proto+ ' '+ doEraseProto+ ' '+ mode+ ' '+ envVarNames
		if debug:
			print 'envVarNames:',envVarNames
			print 'vars to set:'
			for var in envVarsToSet.keys():
				if not var in os.environ.keys():
					print var, ' not in os.environ.keys()'
				else:
					print var,'=', os.environ.get(var)
			print 'end: vars to set'
			print 'cmd:',cmd
		rc= os.system( cmd) # TBD: change to catch errors, or better to be implemented in this script
		if rc != 0:
			error( 'unproto failed for: '+proto)

	# If there is no FileBaseDirectory entry:
	if 'FileBaseDirectory' in envVarsToSet.keys():
		dirs=[servletBaseDir]
	else:
		dirs=[servletsXmlDir, servletBaseDir]

	# chown dirs to frontierUser (default: dbfrontier)
	for dir in dirs:
		#Better: also set groupname: os.system( 'chown -R '+ frontierUser + ':'+ groupname+ ' '+ dir )	
		os.system( 'chown -R '+ frontierUser+ ' '+ dir)	

	# Verify that passwordsFile belongs to correct user and has correct mode
	os.system( 'chown '+ frontierUser+ ' '+ passwordsFile)	
	os.system( 'chmod 600 '+ passwordsFile)	
	# Let user (only) edit /etc/tomcat/servlets.conf
	os.system( 'chown '+ frontierUser+ ' '+ servletsFile)	
	os.system( 'chmod 644 '+ servletsFile)
	# No one to edit /etc/tomcat/servlets.confConstants
	os.system( 'chmod 444 '+ servletsConstantsFile)

	if 'FileBaseDirectory' in envVarsToSet.keys():
		os.system( 'mv '+files4proto['configPropertiesSmallFiles']+' '+files4proto['configProperties'])

def removeIfExists( file):
	if os.path.isfile( file):
		os.remove( file)

def handleUseCopyOfTnsnames():
	if 'defaults' not in servlets.sections():
		useCopyOfTnsnames= False
	else:
		useCopyOfTnsnames='useCopyOfTnsnames' in servlets.options( 'defaults')
	if debug:
		print 'useCopyOfTnsnames:', useCopyOfTnsnames

	updatetnsnamesScriptBase='updatetnsnames.sh'
	updatetnsnamesScript=tomcatBaseDir+'/etc/'+updatetnsnamesScriptBase

	# Handle cron: add/remove line, if needed
	cronFile='/var/spool/cron/'+frontierUser
	cronHasUpdatetnsRecord= os.system('grep '+updatetnsnamesScriptBase+' '+cronFile)==0
	if (useCopyOfTnsnames and not cronHasUpdatetnsRecord): # Add a record
		cronRecord='0 9 * * * '+updatetnsnamesScript
		cmdAdd= 'echo "'+cronRecord +'">>'+cronFile
		if debug:
			print 'cmdAdd',cmdAdd
		os.system( cmdAdd)
	else:
		if (not useCopyOfTnsnames and cronHasUpdatetnsRecord): #Remove the record
			cmdRemove="sed -i '/"+updatetnsnamesScriptBase+"/d' "+cronFile
			if debug:
				print 'cmdRemove',cmdRemove
			os.system( cmdRemove)

	# Run updatetnsnames.sh, to verify that an initial copy of the crontfile exists 
	if (useCopyOfTnsnames):
		os.system( updatetnsnamesScript)

	# For the following command at frontier-tomcat:
	#   Without updatetnsnames: export CATALINA_OPTS="-Dsun.net.inetaddr.ttl=300 -Doracle.net.tns_admin=/etc"
	#   With    updatetnsnames: export CATALINA_OPTS="-Dsun.net.inetaddr.ttl=300 -Doracle.net.tns_admin="+tomcatBaseDir+"/etc"
	# For this end, the exit code of this script indicates if useCopyOfTnsnames
	return useCopyOfTnsnames

def main():
	global servletBaseDir, servletClassesDir, envVarsToSet 

	operation='install'
	for file in metaConfigFiles:
		if not os.path.isfile( file):
			error('File must exist prior to rpm installation: '+ file)

	if debug:
		print 'operation:',operation
	files4proto={}

	# get WEB-INF/lib/ojdbc5_g.jar if missing
	commonLibDir= tomcatSymbolicLink+ '/lib'
	mkdirIfMissing(commonLibDir)
	os.chdir(commonLibDir)
	if not os.path.isfile( jarFile):
		shutil.copyfile( str(tomcatBaseDir)+'lib/'+jarFile , commonLibDir+'/'+jarFile)

	for servlet in servlets.sections():
		envVarsToSet={}
		# Directories
		servletBaseDir= tomcatSymbolicLink +'/webapps/'+servlet
		servletWebInfDir= servletBaseDir+ '/WEB-INF' # For: web.xml
		servletClassesDir= servletWebInfDir+ '/classes' # For: config.properties

		files4proto['servletXml']= servletsXmlDir+'/'+servlet+'.xml'
		files4proto['configProperties']= servletClassesDir+'/config.properties'
		files4proto['configPropertiesSmallFiles']= servletClassesDir+'/config.propertiesSmallFiles'
		files4proto['webXml']= servletWebInfDir+'/web.xml'

		if debug:
			print 'servlet:', servlet
		if operation=='install':
			createServletXmlFile( servlet, files4proto)

	useCopyOfTnsnames= handleUseCopyOfTnsnames()

	sys.exit( useCopyOfTnsnames)
	
if __name__ == "__main__":
    main()
